/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.controllers.inputs;

/**
 * Simple class to store the mouse cursor position and the scroll amount.
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class Mouse {
    
    /**
     * Cursor location at the screen in the X axis.
     */
    private final int x;
    
    /**
     * Cursor location at the screen in the Y axis.
     */
    private final int y;
    
    /**
     * Mouse scroll amount.
     */
    private final float scroll;

    /**
     * Create a mouse object by specifying its location and scroll.
     * @param x Cursor location at the screen in the X axis.
     * @param y Cursor location at the screen in the Y axis.
     * @param scroll Mouse scroll amount.
     */
    public Mouse(final int x, final int y, final float scroll) {
        this.x = x;
        this.y = y;
        this.scroll = scroll;
    }

    /**
     * Get the Cursor location at the screen in the X axis.
     * @return 
     */
    public final int getX() {
        return x;
    }

    /**
     * Get the Cursor location at the screen in the Y axis.
     * @return 
     */
    public final int getY() {
        return y;
    }

    /**
     * Get the scroll amount (never used).
     * @return 
     */
    public final float getScroll() {
        return scroll;
    }
    
}
