/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.views.jvoxelfilter;

import java.awt.Cursor;
import java.awt.Dimension;
import java.util.Stack;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class JVoxelFilter extends javax.swing.JPanel {

    private float minValue;
    private float maxValue;
    private Stack<JVoxelFilterListener> listeners;
    
    /** Creates new form JVoxelFilter */
    public JVoxelFilter() {
        initComponents();
        jLabelModified.setVisible(false);
        this.listeners = new Stack();
        this.jPanelBody.setVisible(false);
        jButtonRetract.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/go-previous-rtl.png")));
        setPreferredSize(new Dimension(220, 44));
    }
    
    /**
     * Change the title of this filtered property
     * @param title 
     */
    public final void setTitle(final String title) {
        this.jLabelTitle.setText(title);
    }
    
    public final void setPropertyMinAndMax(final float min, final float max) {
        this.minValue = min;
        this.maxValue = max;
        
        /* Update the values of the text fields */
        jFormattedTextFieldMin.setText(Float.toString(min));
        jFormattedTextFieldMax.setText(Float.toString(max));
    }
    
    /**
     * Add a listener
     * @param listener 
     */
    public final void addVoxelFilterListener(final JVoxelFilterListener listener) {
        listeners.push(listener);
    }
    
    /**
     * Update this filtered property.
     */
    private void updateProperty() {
        /* Update the visibility of the "modified" icon 
        and the reset button state */
        if (jSliderMin.getValue() > 0) {
            jLabelModified.setVisible(true);
            jButtonReset.setEnabled(true);
        } else if (jSliderMax.getValue() < 100) {
            jLabelModified.setVisible(true);
            jButtonReset.setEnabled(true);
        } else if (jCheckBoxNaN.isSelected()) {
            jLabelModified.setVisible(true);
            jButtonReset.setEnabled(true);
        } else {
            jLabelModified.setVisible(false);
            jButtonReset.setEnabled(false);
        }
        
        /* Call "onChange" function of each listener */
        float min = minValue + (maxValue - minValue) * (float) jSliderMin.getValue() / 100;
        float max = minValue + (maxValue - minValue) * (float) jSliderMax.getValue() / 100;
        final boolean NaN = jCheckBoxNaN.isSelected();
        
        /* Round the min and max if needed */
        if (maxValue - minValue > 1f && maxValue % 1f == 0 && minValue % 1f == 0) {
            min = Math.round(min);
            max = Math.round(max);
        }
        
        for (JVoxelFilterListener listener : listeners) {
            listener.onChange(new JVoxelFilterEvent(min, max, NaN));
        }
    }
    
    public final void reset() {
        if (jSliderMin.getValue() != 0 || jSliderMax.getValue() != 100 || jCheckBoxNaN.isSelected()) {
            /* Reset the min & max sliders */
            jSliderMin.setValue(0);
            jSliderMax.setValue(100);
            jCheckBoxNaN.setSelected(false);

            /* Update this filtered property */
            updateProperty();
        }
    }

    /** This method is called from within the constructor to initialize the
     * form. WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanelHeader = new javax.swing.JPanel();
        jPanelTitle = new javax.swing.JPanel();
        jButtonRetract = new javax.swing.JButton();
        jLabelTitle = new javax.swing.JLabel();
        jLabelModified = new javax.swing.JLabel();
        jPanelReset = new javax.swing.JPanel();
        jButtonReset = new javax.swing.JButton();
        jPanelBody = new javax.swing.JPanel();
        jPanelMin = new javax.swing.JPanel();
        jLabelMin = new javax.swing.JLabel();
        jSliderMin = new javax.swing.JSlider();
        jFormattedTextFieldMin = new javax.swing.JFormattedTextField();
        jPanelMax = new javax.swing.JPanel();
        jLabelMax = new javax.swing.JLabel();
        jSliderMax = new javax.swing.JSlider();
        jFormattedTextFieldMax = new javax.swing.JFormattedTextField();
        jPanelNaN = new javax.swing.JPanel();
        jLabelNaN = new javax.swing.JLabel();
        jCheckBoxNaN = new javax.swing.JCheckBox();

        setMinimumSize(new java.awt.Dimension(183, 44));
        setPreferredSize(new java.awt.Dimension(220, 150));
        setLayout(new java.awt.BorderLayout());

        jPanelHeader.setBackground(new java.awt.Color(232, 235, 242));
        jPanelHeader.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(153, 153, 153)));
        jPanelHeader.setLayout(new javax.swing.BoxLayout(jPanelHeader, javax.swing.BoxLayout.LINE_AXIS));

        jPanelTitle.setBackground(new java.awt.Color(232, 235, 242));
        jPanelTitle.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jButtonRetract.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/go-bottom.png"))); // NOI18N
        jButtonRetract.setToolTipText("Expand/Unexpand the content of this filter");
        jButtonRetract.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jButtonRetract.setPreferredSize(new java.awt.Dimension(27, 27));
        jButtonRetract.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRetractActionPerformed(evt);
            }
        });
        jPanelTitle.add(jButtonRetract);

        jLabelTitle.setFont(new java.awt.Font("DejaVu Sans", 1, 12)); // NOI18N
        jLabelTitle.setText("<title>");
        jLabelTitle.setToolTipText("The name of the property");
        jPanelTitle.add(jLabelTitle);

        jLabelModified.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/help-about.png"))); // NOI18N
        jLabelModified.setToolTipText("This filtered property has been modified (click reset to return to the default values)");
        jPanelTitle.add(jLabelModified);

        jPanelHeader.add(jPanelTitle);

        jPanelReset.setBackground(new java.awt.Color(232, 235, 242));
        jPanelReset.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jButtonReset.setText("Reset");
        jButtonReset.setToolTipText("Reset this filter to the default values.");
        jButtonReset.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jButtonReset.setEnabled(false);
        jButtonReset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonResetActionPerformed(evt);
            }
        });
        jPanelReset.add(jButtonReset);

        jPanelHeader.add(jPanelReset);

        add(jPanelHeader, java.awt.BorderLayout.PAGE_START);

        jPanelBody.setBorder(null);
        jPanelBody.setLayout(new javax.swing.BoxLayout(jPanelBody, javax.swing.BoxLayout.PAGE_AXIS));

        jPanelMin.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jLabelMin.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jLabelMin.setText("Min ");
        jPanelMin.add(jLabelMin);

        jSliderMin.setMajorTickSpacing(25);
        jSliderMin.setMinorTickSpacing(5);
        jSliderMin.setToolTipText("Change the minimum value of this filtered property");
        jSliderMin.setValue(0);
        jSliderMin.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jSliderMin.setPreferredSize(new java.awt.Dimension(100, 21));
        jSliderMin.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSliderMinStateChanged(evt);
            }
        });
        jSliderMin.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jSliderMinMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jSliderMinMouseReleased(evt);
            }
        });
        jPanelMin.add(jSliderMin);

        jFormattedTextFieldMin.setEditable(false);
        jFormattedTextFieldMin.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0.00"))));
        jFormattedTextFieldMin.setText("0.0");
        jFormattedTextFieldMin.setPreferredSize(new java.awt.Dimension(100, 27));
        jPanelMin.add(jFormattedTextFieldMin);

        jPanelBody.add(jPanelMin);

        jPanelMax.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jLabelMax.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jLabelMax.setText("Max");
        jPanelMax.add(jLabelMax);

        jSliderMax.setToolTipText("Change the maximum value of this filtered property");
        jSliderMax.setValue(100);
        jSliderMax.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jSliderMax.setPreferredSize(new java.awt.Dimension(100, 21));
        jSliderMax.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSliderMaxStateChanged(evt);
            }
        });
        jSliderMax.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                jSliderMaxMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                jSliderMaxMouseReleased(evt);
            }
        });
        jPanelMax.add(jSliderMax);

        jFormattedTextFieldMax.setEditable(false);
        jFormattedTextFieldMax.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter()));
        jFormattedTextFieldMax.setText("1.0");
        jFormattedTextFieldMax.setPreferredSize(new java.awt.Dimension(100, 27));
        jPanelMax.add(jFormattedTextFieldMax);

        jPanelBody.add(jPanelMax);

        jPanelNaN.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));

        jLabelNaN.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/mail-reply-sender.png"))); // NOI18N
        jPanelNaN.add(jLabelNaN);

        jCheckBoxNaN.setText("Hide \"NaN\" values");
        jCheckBoxNaN.setToolTipText("Choose if the NaN values of this filtered property will be hiden or not.");
        jCheckBoxNaN.setCursor(new java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
        jCheckBoxNaN.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxNaNActionPerformed(evt);
            }
        });
        jPanelNaN.add(jCheckBoxNaN);

        jPanelBody.add(jPanelNaN);

        add(jPanelBody, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonRetractActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRetractActionPerformed
        /* Invert the visibility of the content */
        this.jPanelBody.setVisible(!jPanelBody.isVisible());
        
        /* Resize this panel */
        if (jPanelBody.isVisible()) {
            jButtonRetract.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/go-bottom.png")));
            setPreferredSize(new Dimension(220, 150));
        } else {
            jButtonRetract.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/go-previous-rtl.png")));
            setPreferredSize(new Dimension(220, 44));
        }
    }//GEN-LAST:event_jButtonRetractActionPerformed

    private void jButtonResetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonResetActionPerformed
        reset();
    }//GEN-LAST:event_jButtonResetActionPerformed

    private void jSliderMinStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSliderMinStateChanged
        /* Display the value of the slider in the text field */
        final float sliderV = (float) jSliderMin.getValue() / 100;
        final float value = minValue + (maxValue - minValue) * sliderV;
        
        if (maxValue - minValue > 1f && maxValue % 1f == 0 && minValue % 1f == 0) {
            jFormattedTextFieldMin.setText(Float.toString(Math.round(value)));
        } else {
            jFormattedTextFieldMin.setText(Float.toString(value));
        }
    }//GEN-LAST:event_jSliderMinStateChanged

    private void jSliderMaxStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSliderMaxStateChanged
        /* Display the value of the slider in the text field */
        final float sliderV = (float) jSliderMax.getValue() / 100;
        final float value = minValue + (maxValue - minValue) * sliderV;
        
        if (maxValue - minValue > 1f && maxValue % 1f == 0 && minValue % 1f == 0) {
            jFormattedTextFieldMax.setText(Float.toString(Math.round(value)));
        } else {
            jFormattedTextFieldMax.setText(Float.toString(value));
        }
    }//GEN-LAST:event_jSliderMaxStateChanged

    private void jSliderMinMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSliderMinMousePressed
        /* Set a moving cursor when the slider is dragged */
        jSliderMin.setCursor(new Cursor(Cursor.MOVE_CURSOR));
    }//GEN-LAST:event_jSliderMinMousePressed

    private void jSliderMinMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSliderMinMouseReleased
        /* Set a handed cursor when the slider is undragged */
        jSliderMin.setCursor(new Cursor(Cursor.HAND_CURSOR));
        
        /* Update this filtered property - 
        This is performed here to improve the performances */
        updateProperty();
    }//GEN-LAST:event_jSliderMinMouseReleased

    private void jSliderMaxMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSliderMaxMousePressed
        /* Set a moving cursor when the slider is dragged */
        jSliderMax.setCursor(new Cursor(Cursor.MOVE_CURSOR));
    }//GEN-LAST:event_jSliderMaxMousePressed

    private void jSliderMaxMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jSliderMaxMouseReleased
        /* Set a handed cursor when the slider is undragged */
        jSliderMax.setCursor(new Cursor(Cursor.HAND_CURSOR));
        
        /* Update this filtered property - 
        This is performed here to improve the performances */
        updateProperty();
    }//GEN-LAST:event_jSliderMaxMouseReleased

    private void jCheckBoxNaNActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxNaNActionPerformed
        /* Update this filtered property */
        updateProperty();
    }//GEN-LAST:event_jCheckBoxNaNActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonReset;
    private javax.swing.JButton jButtonRetract;
    private javax.swing.JCheckBox jCheckBoxNaN;
    private javax.swing.JFormattedTextField jFormattedTextFieldMax;
    private javax.swing.JFormattedTextField jFormattedTextFieldMin;
    private javax.swing.JLabel jLabelMax;
    private javax.swing.JLabel jLabelMin;
    private javax.swing.JLabel jLabelModified;
    private javax.swing.JLabel jLabelNaN;
    private javax.swing.JLabel jLabelTitle;
    private javax.swing.JPanel jPanelBody;
    private javax.swing.JPanel jPanelHeader;
    private javax.swing.JPanel jPanelMax;
    private javax.swing.JPanel jPanelMin;
    private javax.swing.JPanel jPanelNaN;
    private javax.swing.JPanel jPanelReset;
    private javax.swing.JPanel jPanelTitle;
    private javax.swing.JSlider jSliderMax;
    private javax.swing.JSlider jSliderMin;
    // End of variables declaration//GEN-END:variables
}
